'use strict';

var _utils = require('./utils');

var _template = require('./template.manager');

/**
 * @license
 * Copyright Opensoft. All Rights Reserved.
 *
 * Use of this source code is governed by an Apache 2 license that can be
 * found in the LICENSE file at https://bitbucket.org/opensoftgitrepo/lightweightform/src/develop/LICENSE
 */
var _ = {
  createSubDirectories: function createSubDirectories(rootDir, groupId, title) {
    var template = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 'common';

    var packageDir = groupId.replace(/\./g, '/');
    var sourcesDir = rootDir + 'src/main/kotlin/' + packageDir;
    (0, _utils.createDirectory)('' + sourcesDir);
    var testSourcesDir = rootDir + 'src/test/kotlin/' + packageDir;
    (0, _utils.createDirectory)('' + testSourcesDir);
    // write readme
    (0, _utils.writeFile)(rootDir + '/README.md', (0, _template.loadReadme)({
      template: template,
      message: title + ' sources'
    }));
  }
};

module.exports = {
  createMavenProject: function createMavenProject(rootDir, commonDirectory, commonJsDirectory, commonJvmDirectory, projectName, options) {
    (0, _utils.writeFile)(rootDir + '/pom.xml', (0, _template.loadPomParent)({
      groupId: options.groupid,
      artifactId: options.artifactid,
      appName: projectName
    }));

    (0, _utils.createDirectory)('' + commonDirectory);
    (0, _utils.writeFile)(commonDirectory + '/pom.xml', (0, _template.loadCommonPom)({
      parentGroupId: options.groupid,
      parentArtifactId: options.artifactid
    }));
    _.createSubDirectories(commonDirectory + '/', options.groupid, 'Common');

    (0, _utils.createDirectory)('' + commonJsDirectory);
    (0, _utils.writeFile)(commonJsDirectory + '/pom.xml', (0, _template.loadCommonJsPom)({
      parentGroupId: options.groupid,
      parentArtifactId: options.artifactid
    }));
    _.createSubDirectories(commonJsDirectory + '/', options.groupid, 'Common-js', 'common-js');

    (0, _utils.createDirectory)('' + commonJvmDirectory);
    (0, _utils.writeFile)(commonJvmDirectory + '/pom.xml', (0, _template.loadCommonJvmPom)({
      parentGroupId: options.groupid,
      parentArtifactId: options.artifactid
    }));
    _.createSubDirectories(commonJvmDirectory + '/', options.groupid, 'Common-jvm', 'common-jvm');
  }
};