'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * @license
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Copyright Opensoft. All Rights Reserved.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Use of this source code is governed by an Apache 2 license that can be
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * found in the LICENSE file at https://bitbucket.org/opensoftgitrepo/lightweightform/src/develop/LICENSE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */


var _utils = require('./utils');

var _template = require('./template.manager');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var SchemaManager = function () {
  function SchemaManager() {
    _classCallCheck(this, SchemaManager);
  }

  _createClass(SchemaManager, [{
    key: 'createSchemaModule',

    /**
     * Create schema beans in the common maven module
     * @param directory
     * @param basePackage
     * @param schema
     * @param name
     * @param hideName
     * @param isRootLevel
     * @private
     */
    value: function createSchemaModule(directory, basePackage, schema, name, hideName, isRootLevel) {
      (0, _utils.createDirectory)(directory);
      var className = (0, _utils.camelcase)(name);
      /*
           * Get children context
           *  - ListSchemas - tables
           *  - RecordSchemas - Forms
           *  - otherwise - FieldSchemas
           */
      var recordSchemas = [];
      var importPackages = [];
      var childElements = this._getFieldSchemas(schema);
      Object.keys(childElements).forEach(function (fieldName) {
        var fieldClass = (0, _utils.camelcase)(fieldName);
        childElements[fieldName].type = (0, _utils.camelcase)(childElements[fieldName].type) + 'Schema';
        var isListSchema = childElements[fieldName].type === 'ListSchema';
        var isRecordSchema = childElements[fieldName].type === 'RecordSchema';
        var isNumberSchema = childElements[fieldName].type === 'NumberSchema';
        var fieldBasePackage = '' + basePackage + (isRecordSchema ? '.' + fieldName + '.' : '.') + fieldClass;
        var importSchemaPackage = fieldBasePackage + 'Schema';
        var importEntryStoragePackage = '';
        if (isListSchema) {
          importSchemaPackage = basePackage + '.' + fieldName.toLowerCase() + '.' + fieldClass + 'EntrySchema';
          importEntryStoragePackage = (basePackage + '.' + fieldName.toLowerCase() + '.' + fieldClass + 'Entry').replace('.schema.', '.storage.');
          importPackages.push(importEntryStoragePackage);
          if (importPackages.indexOf('pt.lightweightform.kotlin.schema.ListSchema') === -1) {
            importPackages.push('pt.lightweightform.kotlin.schema.ListSchema');
          }
        } else if (isRecordSchema) {
          // do nothing
        } else {
          importSchemaPackage = 'pt.lightweightform.kotlin.schema.' + childElements[fieldName].type;
        }
        if (importPackages.indexOf(importSchemaPackage) === -1) {
          importPackages.push(importSchemaPackage);
        }
        recordSchemas.push(_extends({
          fieldName: fieldName,
          fieldClass: (0, _utils.camelcase)(fieldClass) + 'Schema',
          isRecordSchema: isRecordSchema,
          isListSchema: isListSchema,
          isNumberSchema: isNumberSchema,
          entrySchemaClass: fieldClass + 'EntrySchema',
          entryStorageClass: fieldClass + 'Entry',
          validationContext: hideName ? '' : fieldName
        }, childElements[fieldName]));
      });

      /*
               * Create object with all context necessary to generate class
               */
      var context = {
        name: hideName ? '' : name,
        package: '' + basePackage,
        storageTypeImport: (basePackage + '.' + className).replace('.schema.', '.storage.'),
        schemaType: className + 'Schema',
        storageType: className,
        isRequired: schema.isRequired,
        fields: recordSchemas,
        importPackages: importPackages,
        isRootLevel: isRootLevel
      };
      _utils.logger.debug('adding ' + context.schemaType + '...');
      (0, _utils.writeFile)(directory + '/' + className + 'Schema.kt', (0, _template.loadRecordSchema)(context));

      /**
       * Iterate over all record's children and create their subclasses
       */
      this._processChildrenSchema(recordSchemas, directory, basePackage, childElements);
    }
  }, {
    key: '_getFieldSchemas',
    value: function _getFieldSchemas(schema) {
      return schema.type === 'ListSchema' ? schema.elementsSchema.fieldsSchemas : schema.fieldsSchemas;
    }
  }, {
    key: '_processChildrenSchema',
    value: function _processChildrenSchema(recordSchemas, directory, basePackage, childElements) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = recordSchemas[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var field = _step.value;

          if (field.isRecordSchema) {
            this.createSchemaModule(directory + '/' + field.fieldName.toLowerCase(), basePackage + '.' + field.fieldName.toLowerCase(), childElements[field.fieldName], field.fieldName, false, false);
          }
          if (field.isListSchema) {
            this.createSchemaModule(directory + '/' + field.fieldName.toLowerCase(), basePackage + '.' + field.fieldName.toLowerCase(), childElements[field.fieldName], field.fieldName + 'Entry', true, false);
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }
  }]);

  return SchemaManager;
}();

module.exports = {
  processSchema: function processSchema(rootDirectory, options, schema) {
    var commonPackageRootDirectory = '' + rootDirectory + options.groupid.replace(/\./g, '/');
    (0, _utils.createDirectory)(commonPackageRootDirectory + '/schema');
    new SchemaManager().createSchemaModule(commonPackageRootDirectory + '/schema', options.groupid + '.schema', schema, options.artifactid, true, true);
  }
};