'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /**
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * @license
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Copyright Opensoft. All Rights Reserved.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * Use of this source code is governed by an Apache 2 license that can be
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      * found in the LICENSE file at https://bitbucket.org/opensoftgitrepo/lightweightform/src/develop/LICENSE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      */


var _utils = require('./utils');

var _template = require('./template.manager');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var StorageManager = function () {
  function StorageManager() {
    _classCallCheck(this, StorageManager);
  }

  _createClass(StorageManager, [{
    key: 'createStorageModule',

    /**
     * Create storage beans in the common maven module
     * @param directory
     * @param basePackage
     * @param schema
     * @param name
     * @private
     */
    value: function createStorageModule(directory, basePackage, schema, name) {
      (0, _utils.createDirectory)(directory);
      var fields = [],
          importPackages = [];
      var childElements = this._getFieldSchemas(schema);
      Object.keys(childElements).forEach(function (fieldName) {
        var isNullable = childElements[fieldName].isNullable;
        var fieldType = null,
            isList = false,
            isRecord = false;
        if (childElements[fieldName].type === 'RecordSchema') {
          fieldType = (0, _utils.camelcase)(fieldName);
          isRecord = true;
        } else if (childElements[fieldName].type === 'ListSchema') {
          fieldType = (0, _utils.camelcase)(fieldName) + 'Entry';
          isList = true;
        } else if (childElements[fieldName].type === 'NumberSchema') {
          fieldType = 'Long';
        } else {
          fieldType = childElements[fieldName].type.replace('Schema', '');
        }
        var importPackage = isRecord || isList ? basePackage + '.' + fieldName.toLowerCase() + '.' + fieldType : undefined;
        fields.push({
          fieldName: fieldName,
          fieldType: fieldType,
          isNullable: isNullable,
          isList: isList,
          isRecord: isRecord
        });
        if (importPackage && importPackages.indexOf(importPackage) === -1) {
          importPackages.push(importPackage);
        }
      });
      /*
               * Create object with all context necessary to generate class
               */
      var context = {
        package: '' + basePackage,
        className: (0, _utils.camelcase)(name),
        fields: fields,
        importPackages: importPackages
      };

      _utils.logger.debug('adding ' + context.className + '...');
      (0, _utils.writeFile)(directory + '/' + (0, _utils.camelcase)(name) + '.kt', (0, _template.loadRecordStorage)(context));

      /*
           * Iterate over all record's children and create their subclasses
           */
      this._processChildrenStorage(fields, directory, basePackage, childElements);
    }
  }, {
    key: '_getFieldSchemas',
    value: function _getFieldSchemas(schema) {
      return schema.type === 'ListSchema' ? schema.elementsSchema.fieldsSchemas : schema.fieldsSchemas;
    }
  }, {
    key: '_processChildrenStorage',
    value: function _processChildrenStorage(fields, directory, basePackage, childElements) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = fields[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var field = _step.value;

          if (field.isRecord) {
            this.createStorageModule(directory + '/' + field.fieldName.toLowerCase(), basePackage + '.' + field.fieldName.toLowerCase(), childElements[field.fieldName], field.fieldName);
          }
          if (field.isList) {
            this.createStorageModule(directory + '/' + field.fieldName.toLowerCase(), basePackage + '.' + field.fieldName.toLowerCase(), childElements[field.fieldName], field.fieldName + 'Entry');
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }
    }
  }]);

  return StorageManager;
}();

module.exports = {
  processStorage: function processStorage(rootDirectory, options, schema) {
    var commonPackageRootDirectory = '' + rootDirectory + options.groupid.replace(/\./g, '/');
    (0, _utils.createDirectory)(commonPackageRootDirectory + '/storage');
    new StorageManager().createStorageModule(commonPackageRootDirectory + '/storage', options.groupid + '.storage', schema, options.artifactid);
  }
};